@@ -118,7 +118,7 @@ fxt_blockev_t fxt_blockev_enter(fxt_t fxt)
 {
 	fxt_blockev_t evs;
 
-	if (!(evs = malloc(sizeof(*evs)))<0) goto out;
+	if (!(evs = malloc(sizeof(*evs)))) goto out;
 	evs->fxt=fxt;
 
 	if (fxt_fill_blockev(fxt, evs) < 0) goto outmalloc;
