@@ -1,6 +1,6 @@
 /*	fut_setup.c */
 /*
- * Copyright (C) 2012, 2016 Université Bordeaux 1
+ * Copyright (C) 2012, 2016, 2024 Université Bordeaux 1
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -81,14 +81,14 @@ int main(int argc, char *argv[])
 #ifdef __MINGW32__
 					GetCurrentThreadId()
 #else
-					pthread_self()
+					(uintptr_t) pthread_self()
 #endif
 					);
 	fut_keychange(FUT_ENABLE, FUT_KEYMASKALL,
 #ifdef __MINGW32__
 						GetCurrentThreadId()
 #else
-						pthread_self()
+						(uintptr_t) pthread_self()
 #endif
 						);
 
