@@ -77,7 +77,7 @@ char	*fut_bufptr[2];
 int	fut_curbuf; /* buffer currently in use */
 
 /*	number of contiguous allocated bytes pointed to by fut_bufptr */
-unsigned int	fut_nallocated = 0;
+uint64_t fut_nallocated = 0;
 
 /* filename of the output trace */
 static char *fut_filename = NULL;
@@ -96,7 +96,9 @@ pthread_mutex_t fut_flush_lock = PTHREAD_MUTEX_INITIALIZER;
 
 static int already_flushed = 0;
 
-void fut_set_filename(char* filename)
+void (*fut_flush_callback)(void) = NULL;
+
+void fut_set_filename(const char* filename)
 {
   pthread_mutex_lock(&fut_flush_lock);
   if(fut_filename) {
@@ -186,9 +188,9 @@ void disable_fut_flush()
 	includes mallocing the buffer to hold the trace.
 	returns number of bytes allocated if all ok, else a negative error code.
 */
-int fut_setup( unsigned int nlongs, unsigned int keymask, unsigned int threadid )
+int64_t fut_setup( uint64_t nlongs, unsigned int keymask, unsigned int threadid )
 	{
-	unsigned int	nbytes;
+	uint64_t	nbytes;
 	unsigned long	*iptr;
 
 
@@ -262,6 +264,12 @@ int fut_setup( unsigned int nlongs, unsigned int keymask, unsigned int threadid
 	return nlongs;
 	}
 
+int64_t fut_setup_flush_callback( uint64_t nlongs, unsigned int keymask, unsigned int threadid, void (*flush_callback)(void) )
+	{
+	fut_flush_callback = flush_callback;
+
+	return fut_setup(nlongs, keymask, threadid);
+	}
 
 /*	called repeatedly to restart tracing.
 	returns previous value of fut_active if all ok, else a negative error code.
@@ -327,9 +335,9 @@ int fut_done( void )
 /*	called to reset tracing to refill the entire buffer again.
 	returns number of bytes in buffer if all ok, else a negative error code.
 */
-int fut_reset( unsigned int keymask, unsigned int threadid )
+int64_t fut_reset( unsigned int keymask, unsigned int threadid )
 	{
-	unsigned int	nlongs;
+	uint64_t nlongs;
 
 
 	/* paranoia, so nobody waits for us while we are in here */
@@ -363,9 +371,9 @@ int fut_reset( unsigned int keymask, unsigned int threadid )
 /*	called repeatedly to copy current buffer into user space.
 	returns nlongs >= 0 if all ok, else a negative error code.
 */
-int fut_getbuffer( int *nlongs, unsigned long **buffer, unsigned long* first_slot, unsigned long *next_slot )
+uint64_t fut_getbuffer( uint64_t *nlongs, unsigned long **buffer, unsigned long* first_slot, unsigned long *next_slot )
 	{
-	int				local_nlongs = 0;
+	uint64_t local_nlongs = 0;
 
 
 	if( fut_bufptr[0] == NULL )
@@ -387,11 +395,12 @@ out:
 	return local_nlongs;
 	}
 
-int fut_flush( char* filename, void *first_slot, void* next_slot, int record_flush_events )
+uint64_t fut_flush( const char* filename, void *first_slot, void* next_slot, int record_flush_events )
 {
 
-  int n, nlongs=0;
-  long size;
+  uint64_t n=0;
+  uint64_t nlongs = 0;
+  size_t size;
   unsigned long *copy=NULL;
   static off_t begin = 0;
 
@@ -405,11 +414,11 @@ int fut_flush( char* filename, void *first_slot, void* next_slot, int record_flu
   if(fut_fd == -1) {
 
     /* First time fut_flush is called. Open the output file and write headers */
-    if( fut_filename == NULL )
-      fut_filename = filename ;
+    if( fut_filename == NULL && filename )
+      fut_filename = strdup(filename);
 
     if( fut_filename == NULL )
-      fut_filename = DEFAULT_TRACE_FILE;
+      fut_filename = strdup(DEFAULT_TRACE_FILE);
 
     /* we can't open the file using the O_NONBLOCK option since after a call to write
      * we are going to modify the buffer. If O_NONBLOCK is set, modifying the buffer while
@@ -441,10 +450,22 @@ int fut_flush( char* filename, void *first_slot, void* next_slot, int record_flu
   if(record_flush_events)
     __fut_record_event(&start_flush_event, fut_getstamp(), FUT_CODE(FUT_START_FLUSH_CODE, 0));
 
-  /* dump the buffer to disk */
-  if( write(fut_fd, (void *)copy, size) < 0 ) {
-    perror("write buffer");
-  }
+  /* dump the buffer to disk
+   * since write syscall can write at most ~2GB of data (see NOTES of `man 2 write`),
+   * we need to loop while whole buffer isn't flushed, if the buffer size if more than 2 GB.
+   */
+  ssize_t written = 0;
+  size_t total_written = 0;
+  size_t to_write = size;
+  do {
+    written = write(fut_fd, (void *)copy + total_written, size - total_written);
+    if( written < 0 ) {
+      perror("write buffer");
+    }
+    else {
+      total_written += written;
+    }
+  } while(written >= 0 && total_written < size);
 
   fxt_trace_user_raw_t stop_flush_event;
   if(record_flush_events) {
@@ -480,14 +501,15 @@ int fut_flush( char* filename, void *first_slot, void* next_slot, int record_flu
   return nlongs;
 }
 
-int fut_endup( char *filename )
+uint64_t fut_endup( const char *filename )
 	{
 
 	  /* stop all futher tracing */
 	  fut_active = 0;
+	  allow_fut_flush = 0;
 
 	  /* dump the buffer to disk */
-	  int nlongs = fut_flush(filename, fut_first_slot, fut_next_slot, 0);
+	  uint64_t nlongs = fut_flush(filename, fut_first_slot, fut_next_slot, 0);
 	  if(nlongs < 0)
 	    return nlongs;
 
