@@ -48,7 +48,7 @@ int fxt_get_cpu_info( fxt_t fxt )
 		free(fxt->infos.mhz);
 	fxt->infos.ncpus = 0;
 
-#ifndef __MINGW32__
+#if ! (defined(__MINGW32__) || defined(__APPLE__))
 	if( (fptr=fopen(PROC_FILE, "r")) == NULL )
 		{
 		perror("fopen");
@@ -58,7 +58,7 @@ int fxt_get_cpu_info( fxt_t fxt )
 		fxt->infos.mhz = malloc(sizeof(*fxt->infos.mhz));
 		fxt->infos.mhz[0] = 1000;
 		return fxt->infos.ncpus = 1;
-#ifndef __MINGW32__
+#if ! (defined(__MINGW32__) || defined(__APPLE__))
 		}
 
 	for( ; ; )
